'' +--------------------------------------------------------------------------+
'' |  Cluso's:       Propeller P8X32A Simple Test                       v0.10 |
'' +--------------------------------------------------------------------------+
'' |  Purpose:       Test Propeller P8X32A FPGA                               |
'' |  Authors:       (c)2014 "Cluso99" (Ray Rodrick)                          |
'' |  License:       MIT License (for this program only)                      |
'' |                   - See end of file for terms of use                     |
'' |  WARNING: The Verilog FPGA code is Copyright by Parallax www.parallax.com|
'' +--------------------------------------------------------------------------+
''
'' RR20140827  v0.10    Commence
'' RR20140904   008     AUGDS works with 20140904e.zip fgpa code
'' RR20140906   010     AUGS #23bits test
'' RR20140907   014     wkg  (some bugs)
'' RR20140909   021     still a bug where AUGxx causes a run-on (disobeys count!)
'' RR20140909   025     wkg w 20140909c_augxx fpga




CON
        _clkmode = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
        _xinfreq = 5_000_000
'       _xinfreq = 6_500_000                                                    ' 104MHz


        RAMBASE  = $0000                        ' RAM               $0000..7FFF  
        ROMBASE  = $8000                        ' ROM               $8000..FFFF
        RAMSIZE  = 8192 'longs                  ' RAM:              $0000..7FFF                       
        FONTSIZE = 4096 'longs                  ' ROM: FONT         $8000..BFFF                       
        LOGSIZE  = 1024 'longs                  ' ROM: LOG          $C000..CFFF 2048 words            
        ALOGSIZE = 1024 'longs                  ' ROM: ANTILOG      $D000..DFFF 2048 words            
        SINESIZE = 1025 'longs                  ' ROM: SINE         $E000..F002 2049 words + 1 filler 
'       INTERPSZ =                              ' ROM: Interpreter  $F004..F7A3  488 longs  (scrambled)           
'       BOOTERSZ =                              ' ROM: Booter       $F800..FB93  229 longs  (scrambled)           
'       RUNNERSZ =                              ' ROM: Runner       $FF70..FFFF   36 longs                     

 
OBJ
  fdx   : "FullDuplexSerial"

VAR
  long  cog
  long  flag          ' \\
  long  data[16]      ' //
        
PUB Start | i

  waitcnt(clkfreq*5 + cnt)
  fdx.Start(31, 30, %0000, 115200)
  waitcnt(clkfreq/10 + cnt)
  fdx.tx(0)                                             ' cls
  fdx.tx(1)                                             ' home
  waitcnt(clkfreq/10 + cnt)
  fdx.Str(string("Test begin...",13,13))
  
  flag:=1                                               ' hold pasm cog
  cog := cognew(@entry, @flag) + 1                      ' start the new cog now, assembly cog at "entry" label.
  waitcnt(clkfreq/2 + cnt)
  
  repeat i from 0 to 520
    flag~
    repeat while (flag==0)                              ' wait for pasm cog
    fdx.hex(data[0],8)
    fdx.tx(" ")
    fdx.hex(data[1],8)
    fdx.tx(" ")
    fdx.hex(data[2],8)
    fdx.tx(" ")
    fdx.hex(data[3],8)
    fdx.tx(" ")
    fdx.hex(data[4],3)
    fdx.tx(" ")
    fdx.hex(i,3)
    fdx.tx(13)

  repeat                                                '<===============loop    


CON
_AUGDS  = %000110_0001_0000 << 18
_AUGS   = %000110_010       << 23
'       long  _AUGS  | ($FFFF_FE00 >> 9)                                        ' AUGS  #S    (23bits)
'       long  _AUGS  | %1100_1100_1100_1100_1100_110                            ' AUGS  #S    (23bits)
'       long  _AUGDS | (%00_0000_000 << 9) | (%00_0000_000)                     ' AUGDS #D,#S (9+9bits)

DAT
              org       0                                                       
entry
'' The following are used for testing the USRn instructions...
t0            long      $0000
t1            long      $1111
t2            long      $2222
t3            long      $3333


              add       p0,par                          ' hub ptr to data[0]
              add       p1,par
              add       p2,par
              add       p3,par
              add       p4,par

:wait1        rdlong    tmp,par wz                      ' wait till ready?
        if_nz jmp       #:wait1

{
:kill         jmp       #:skip
:stall        jmp       #:stall
:skip         mov       :kill,0
}

''===================================================================================

' fill upper ram & lower ram (do lower first)
              mov       count,#16 '#511
              movs      :ind,#$050                      ' init the value stored            
              movd      :ind,#$100                      ' init the cog ram base addr      
              movs      :inc,#$080                      ' init the value stored            
              movd      :inc,#$100                      ' init the cog ram base addr      
              nop
:fill
' lower
:ind          mov       0-0,#0-0                        ' mov lower $000++, #$4444_44nn++
              add       :ind,#1                         ' S++ (can overflow into D++)
              add       :ind,h200                       ' D++

'             jmp       #:stepover

' upper
:augds  long  _AUGDS | (%00_0000_001 << 9) | (%00_0000_000)  ' set D=$200 (ie upper cog ram) & S=0
:augs   long  _AUGS  | ($C7CC_0C00 >> 9)                     ' set S=$CCCC_CCxx


:inc          mov       0-0,#0-0                        ' mov upper $200++, #$4444_44nn++
              add       :inc,#1                         ' S++                    
              add       :augs,#8                        ' AUG(S)++8              
              add       :inc,h200                       ' D++

' loop
:stepover     djnz      count,#:fill

''-----------------------------------------------------------------------------------
' readback upper ram & lower ram & special regs $1Fx
              mov       count,#16 '511
              movs      :inx,#$100                      ' init the cog ram base addr
              movs      :iny,#$100                      ' init the cog ram base addr
              movs      :inz,#$1F0                      ' init the cog ram special addr    
              nop
:rd
' upper
              long      _AUGDS | (%00_0000_000 << 9) | (%00_0000_001) ' set D=0 & S=$200 (ie upper cog ram)
:inx          mov       t0,0-0                          ' mov upper $200++ to t0
              add       :inx,#1                         ' S++                    
' lower
:iny          mov       t1,0-0                          ' mov lower $000++ to t1
              add       :iny,#1                         ' S++
' special
:inz          mov       t2,0-0                          ' mov $1F0++ to t2
              add       :inz,#1                         ' S++
              movd      :inz,#t2                        ' ensure wrap S
' send
              call      #send
              djnz      count,#:rd
''-----------------------------------------------------------------------------------
stop1         jmp       #stop1

''===================================================================================
send          wrlong    t0,p0                           ' write to hub...
              wrlong    t1,p1
              wrlong    t2,p2
              wrlong    t3,p3
              wrlong    n,p4
              add       n,#1
              wrlong    one,par                         ' set avail flag
:loop         rdlong    tmp,par wz                      ' wait till ready?
        if_nz jmp       #:loop
send_ret      ret
''-----------------------------------------------------------------------------------

p0      long  4                 ' data[0] == par+4
p1      long  8
p2      long  12
p3      long  16
p4      long  20

one     long  1
count   long  0
fcount  long  0
tmp     long  0
n       long  0
h200    long  1 << 9

        long  0[$1F0-$]
        fit   $1F0
        

DAT
''==============================================================================================================
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}

  